// ProgramCmd.cpp: CProgramCmd Implementation.
//
// This software can be offered for free and used as necessary to aid 
// in your program developments.
//
// RENESAS TECHNOLOGY CORPORATION, RENESAS SOLUTIONS CORPORATION,
// and related original software developers assume no responsibility 
// for any damage or infringement of any third-party's rights, originating 
// in the use of the following software.
// Please use this software under the agreement and acceptance of these conditions. 
//
// Copyright(C)1998(2003) RENESAS TECHNOLOGY CORPORATION AND RENESAS SOLUTIONS CORPORATION
// ALL RIGHTS RESERVED
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "M16Cflsh.h"
#include "ProgramCmd.h"
#include "PreWriteDlg.h"
#include "PrgssDlg.h"
#include "SetCommDlg.h" // add intervals
#include "defining.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProgramCmd::CProgramCmd()
{

}

CProgramCmd::~CProgramCmd()
{

}

BOOL CProgramCmd::exec(void)
{
	BOOL bResult = FALSE;
	CM16CflshApp*		pApp = (CM16CflshApp*)AfxGetApp();
	MSG				msg;
    CSetCommDlg  dlg2;

	// Get Communication object.
	CSerialComm& comObj = GetSerialComm();

	// Get Programm address range.
	unsigned int nStartAddress = 0;
	unsigned int nEndAddress = 0;
	if(!GetFileAddress(&nStartAddress, &nEndAddress, ADDRESS_SIZE))
	{
		return FALSE;
	}
	// get Intervals time
	DWORD ndwintervals = 0;
	if(!dlg2.GetIntervals(&ndwintervals))
	{
		return FALSE;
	}
	// If a start address is larger than an end address, it will end.
	if(!ChkStartToEnd(nStartAddress, nEndAddress))
	{
		AfxGetMainWnd()->MessageBox(GetResString(IDS_SMALESTART), GetResString(IDS_FLASH_TITLE), MB_ICONWARNING);
		return FALSE;
	}

	// write confirm.
	CPreWriteDlg dlg;
	dlg.SetAddress(HexToStr(nStartAddress), HexToStr(nEndAddress));
	int nResponse = dlg.DoModal();
	if(IDCANCEL == nResponse)
	{
		AfxGetMainWnd()->MessageBox(GetResString(IDS_STOP_PROGRAM), GetResString(IDS_FLASH_TITLE));
		return FALSE;
	}

	// Clear status
	BYTE send = 0x50;
	comObj.Write(&send, 1);

	// Wait for Communication.
	Sleep(SLEEP_IN);

	// Prograss dialog setup.
	CProgressDlg* pDlg;
	BOOL bCancel = FALSE;
	pDlg = CProgressDlg::Instance(&bCancel, NULL);
	pDlg->InitialDialog("Program.", "Program", 
							0, (nEndAddress - nStartAddress - 0x0100) / TO_INT, 0);

	// add 99.4.27 for M16C/20 2Mhz
	GetSerialComm().SetTimeout(TIMEOUT_PRG);

	// Write data.
	for(int nSendAddress = nStartAddress; (long)nSendAddress < (long)nEndAddress; nSendAddress += 0x0100)
	{
		// Get hex data from file.
		BYTE byteData[PAGE_SIZE];
		ClearPage(byteData);
		int nPage = nSendAddress / 0x0100;
		GetPage(byteData, nPage);

		// Prepare send data.
		BYTE send[3 + PAGE_SIZE];
		send[0] = 0x41;
		ToSendStyle(nSendAddress, &send[2], &send[1]);
		for(int nIndex = 0; nIndex < PAGE_SIZE; nIndex++)
		{
			send[nIndex + 3] = byteData[nIndex];
		}

		// Send data.
		bResult = comObj.Write(send, 3 + PAGE_SIZE);
	
		// Prograss Bar Update.
		pDlg->SetProgressPos((nSendAddress - nStartAddress) / TO_INT);

		// Implement the event.
  		while (::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
			if (!pApp->PumpMessage()) {
				::PostQuitMessage(0);
				break;
			}
		}
		if(bCancel)
		{
			AfxGetMainWnd()->MessageBox(GetResString(IDS_STOP_PROGRAM), GetResString(IDS_FLASH_TITLE), MB_ICONINFORMATION);
			break;
		}

		// Wait next Communicate.
		//Sleep(SLEEP_IN); // change set Intervals time
		Sleep(ndwintervals);

		// add 99.4.27 for 20 2Mhz
		// Check Status
		bResult = !CheckState(&comObj, "program");
		if(!bResult)
		{
			// AfxGetMainWnd()->MessageBox(GetResString(IDS_PROGRAM_ERROR), GetResString(IDS_FLASH_TITLE), MB_ICONERROR);
			break;
		}
		else
		{
			// AfxGetMainWnd()->MessageBox(GetResString(IDS_PROGRAM_OK), GetResString(IDS_FLASH_TITLE), MB_ICONINFORMATION);
		}
		// add 99.4.27 for 20 2Mhz end

	}

	// add 99.4.27 for 20 2Mhz
	GetSerialComm().SetTimeout(TIMEOUT5);

	BOOL bIsEnd = bCancel;
	pDlg->Done();

	if(!bIsEnd)
	{
		// Check Status
		bResult = !CheckState(&comObj, "program");
		if(!bResult)
		{
			AfxGetMainWnd()->MessageBox(GetResString(IDS_PROGRAM_ERROR), GetResString(IDS_FLASH_TITLE), MB_ICONERROR);
		}
		else
		{
			AfxGetMainWnd()->MessageBox(GetResString(IDS_PROGRAM_OK), GetResString(IDS_FLASH_TITLE), MB_ICONINFORMATION);
		}
	}

	return	bResult;
}
